#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Query planning.
"""

import csv
import pathlib
import typing

from icecream import ic  # type: ignore  # pylint: disable=E0401,W0611

from .cypher import CypherItem


class Query:
    """
Manage the lifecycle of query plan.
    """
    def __init__ (
        self,
        ) -> None:
        """
Constructor.
        """
        self.items: typing.List[CypherItem] = []
        self.bindings: dict = {}


    def load_ast (
        self,
        ast_path: pathlib.Path,
        ) -> None:
        """
Load a TSV file of AST items.
        """
        for row in csv.reader(ast_path.open(), delimiter="\t"):
            item = CypherItem(row)

            # back-link the parent item
            if item.parent >= 0:
                self.items[item.parent].children.append(item.ordinal)

            self.items.append(item)


    def query_plan (  # pylint: disable=R0914
        self,
        ) -> typing.Tuple[str, str, str, str]:
        """
Develop a query plan from the given AST items.
        """
        i = 0

        while i < len(self.items):
            if self.items[i].ast_typestr == "statement":
                break

            i += 1

        query_i = self.items[i].children[0]
        match_i, ret_i = self.items[query_i].children

        ## bindings
        pat_i = self.items[match_i].children[0]
        pat_path_i = self.items[pat_i].children[0]
        node_pat_i = self.items[pat_path_i].children[0]

        ident_i, map_i = self.items[node_pat_i].children
        node_name = self.items[ident_i].literal  # pylint: disable=W0621

        self.bindings[node_name] = map_i

        ## selection
        prop_i, val_i = self.items[map_i].children
        prop_key = self.items[prop_i].literal  # pylint: disable=W0621
        prop_val = self.items[val_i].literal.strip("'")  # pylint: disable=W0621

        ## projection
        proj_i = self.items[ret_i].children[0]
        ident_i = self.items[proj_i].children[0]
        ret_name = self.items[ident_i].literal  # pylint: disable=W0621

        print(self.bindings)

        return (node_name, prop_key, prop_val, ret_name)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""
Cypher ASTs.
"""

import typing


class CypherItem:  # pylint: disable=R0902,R0903
    """
Represent data for one AST tree item in a parsed Cypher query.
    """
    REPR_STR = "@{}[{}]\t{}..{}\t{:<4}{:<10}{:<16} |> {}"

    def __init__ (
        self,
        row: typing.List[str],
        ) -> None:
        """
Constructor.
        """
        ordinal, parent, depth, s0, s1, ast_typestr, literal = row  # pylint: disable=C0103

        self.ordinal: int = int(ordinal)
        self.parent: int = int(parent)
        self.depth: int = int(depth)
        self.s0: int = int(s0)  # pylint: disable=C0103
        self.s1: int = int(s1)  # pylint: disable=C0103
        self.ast_typestr: str = ast_typestr
        self.literal: str = literal
        self.children: typing.List[int] = []


    def __repr__ (
        self,
        ) -> str:
        """
Printed representation for an AST item.
        """
        return self.REPR_STR.format(
            self.ordinal,
            self.parent,
            self.s0,
            self.s1,
            self.depth,
            str(self.children),
            self.ast_typestr,
            self.literal,
            )



# type: ignore

"""
Python packaging.
"""

import importlib.util
import pathlib
import typing

import setuptools


KEYWORDS = [
    "cypher",
    "query optimization",
    "graph query",
    "knowledge graph",
    ]


def parse_requirements_file (filename: str) -> typing.List:
    """read and parse a Python `requirements.txt` file, returning as a list of str"""
    with pathlib.Path(filename).open() as f:  # pylint: disable=C0103
        return [ l.strip().replace(" ", "") for l in f.readlines() ]


if __name__ == "__main__":
    spec = importlib.util.spec_from_file_location("goedwig.version", "goedwig/version.py")
    goedwig_version = importlib.util.module_from_spec(spec)
    spec.loader.exec_module(goedwig_version)
    MIN_PY_VERSION = ".".join([ str(x) for x in goedwig_version.MIN_PY_VERSION ])

    base_packages = parse_requirements_file("requirements.txt")
    docs_packages = parse_requirements_file("requirements-dev.txt")

    setuptools.setup(
        name = "goedwig",
        version = goedwig_version.__version__,
        python_requires = ">=" + MIN_PY_VERSION,

        packages = setuptools.find_packages(exclude=[ "docs", "examples" ]),
        install_requires = base_packages,
        extras_require = {
            "base": base_packages,
            "docs": docs_packages,
            },

        author = "Paco Nathan",
        author_email = "paco@derwen.ai",
        license = "MIT",

        description = "A generic query engine for graph data, based on abstract syntax trees as input.",  # pylint: disable=C0301
        long_description = pathlib.Path("README.md").read_text(),
        long_description_content_type = "text/markdown",

        keywords = ", ".join(KEYWORDS),
        classifiers = [
            "Programming Language :: Python :: 3",
            "License :: OSI Approved :: MIT License",
            "Operating System :: OS Independent",
            "Development Status :: 5 - Production/Stable",
            "Intended Audience :: Developers",
            "Intended Audience :: Education",
            "Intended Audience :: Information Technology",
            "Intended Audience :: Science/Research",
            "Topic :: Scientific/Engineering :: Artificial Intelligence",
            "Topic :: Scientific/Engineering :: Human Machine Interfaces",
            "Topic :: Scientific/Engineering :: Information Analysis",
            ],

        url = "https://github.com/DerwenAI/goedwig",
        project_urls = {
            "Source Code": "https://github.com/DerwenAI/goedwig",
            "Issue Tracker": "https://github.com/DerwenAI/goedwig/issues",
            },

        zip_safe = False,
        )
